@@ -1,7 +1,4 @@
-use std::{
-    ops::Range,
-    sync::{mpsc::Sender, Arc},
-};
+use std::{ops::Range, sync::mpsc::Sender};
 
 use cursive::{Vec2, View};
 
@@ -9,45 +6,40 @@ use crate::{
     align::{AlignAlgorithm, AlignElement},
     backend::{Action, Backend, Cursiv},
     datastruct::{CompVec, DoubleVec, SignedArray},
-    drawer::{
-        get_doublehex_columns, print_bottom_line, print_doublehex_screen, print_doublehex_scrolled,
-        print_title_line, set_doublehex_cursor, CursorActive, CursorState, DoubleHexLine,
-    },
-    utils::PointedFile,
+    drawer::{CursorActive, DoubleHexContext, DoubleHexLine},
+    utils::{FileContent, PointedFile},
 };
 
 /// An unaligned view that is just two files next to each other
 pub struct Unaligned {
-    data: CompVec<u8>,
+    data: CompVec,
     filenames: (String, String),
     index: isize,
-    cursor: CursorState,
+    pub dh: DoubleHexContext,
     cursor_act: CursorActive,
 }
 
 impl Unaligned {
     /// Creates a new view, with the indexes in the files at the cursor
-    pub fn new(left: PointedFile, right: PointedFile, cursor: CursorState) -> Self {
-        let mut index = -(cursor.get_index() as isize);
-        let mut data = CompVec::new(left.content, right.content);
-        index += data.add_left_shift(-(left.index as isize));
-        index += data.add_right_shift(-(right.index as isize));
+    pub fn new(first: PointedFile, second: PointedFile, dh: DoubleHexContext) -> Self {
+        let mut index = -(dh.cursor.get_index() as isize);
+        let mut data = CompVec::new(first.content, second.content);
+        index += data.add_first_shift(-(first.index as isize));
+        index += data.add_second_shift(-(second.index as isize));
         Unaligned {
             data,
-            filenames: (left.name, right.name),
+            filenames: (first.name, second.name),
             index,
-            cursor,
+            dh,
             cursor_act: CursorActive::Both,
         }
     }
     /// Resizes the view without drawing it, returning if anything changed
     pub fn resize(&mut self, dimensions: (usize, usize)) -> bool {
         let (columns, rows) = dimensions;
-        let hex_col = get_doublehex_columns(columns);
-        let old_dimensions = (self.cursor.get_size_x(), self.cursor.get_size_y());
-        // subtract two for top and bottom bars
-        let new_dimensions = (hex_col, rows - 2);
-        self.index += self.cursor.resize(new_dimensions);
+        let old_dimensions = (self.dh.cursor.get_size_x(), self.dh.cursor.get_size_y());
+        let new_dimensions = self.dh.style.get_doublehex_dims(columns, rows);
+        self.index += self.dh.cursor.resize(new_dimensions);
         (old_dimensions) != new_dimensions
     }
     /// Redraws without checking for resize.
@@ -57,7 +49,7 @@ impl Unaligned {
             printer.clear();
         }
         let content = self.get_content();
-        print_doublehex_screen(&content, printer);
+        self.dh.print_doublehex_screen(&content, printer);
         self.set_cursor(printer, self.cursor_act);
         self.print_bars(printer);
         printer.refresh();
@@ -69,45 +61,41 @@ impl Unaligned {
     }
     /// Paints the cursor at the current position
     fn set_cursor<B: Backend>(&self, printer: &mut B, cursor_act: CursorActive) {
-        let cursor_index = self.index + self.cursor.get_index() as isize;
-        set_doublehex_cursor(
-            printer,
-            &self.cursor,
-            cursor_act,
-            self.data.get(cursor_index),
+        let cursor_index = self.index + self.dh.cursor.get_index() as isize;
+        let addr = (
+            self.data.get_first_addr(cursor_index),
+            self.data.get_second_addr(cursor_index),
         );
+        self.dh
+            .set_doublehex_cursor(printer, cursor_act, self.data.get(cursor_index), addr);
     }
     /// Converts the content of the CompVec into DoubleHexLines so they can be displayed
     fn get_content(&self) -> Vec<DoubleHexLine> {
         let mut content = Vec::new();
-        for x in 0..self.cursor.get_size_y() {
+        for x in 0..self.dh.cursor.get_size_y() {
             // address of the nth line
-            let base_addr = (x * self.cursor.get_size_x()) as isize + self.index;
+            let base_addr = (x * self.dh.cursor.get_size_x()) as isize + self.index;
             let address = (
-                self.data.get_left_addr(base_addr),
-                self.data.get_right_addr(base_addr),
+                self.data.get_first_addr(base_addr),
+                self.data.get_second_addr(base_addr),
             );
             let bytes = self
                 .data
-                .get_range(base_addr..base_addr + self.cursor.get_size_x() as isize);
+                .get_range(base_addr..base_addr + self.dh.cursor.get_size_x() as isize);
             content.push(DoubleHexLine { address, bytes });
         }
         content
     }
     /// Prints the top and bottom bar
     fn print_bars<B: Backend>(&self, printer: &mut B) {
-        print_title_line(
-            printer,
-            " unaligned",
-            &self.filenames.0,
-            &self.filenames.1,
-            self.cursor.get_size_x(),
-        );
-        print_bottom_line(
-            printer,
-            self.cursor.get_size_x(),
-            self.cursor.get_size_y() + 1,
+        self.dh
+            .print_title_line(printer, " unaligned", &self.filenames.0, &self.filenames.1);
+        let cursor_index = self.index + self.dh.cursor.get_index() as isize;
+        let addr = (
+            self.data.get_first_addr(cursor_index),
+            self.data.get_second_addr(cursor_index),
         );
+        self.dh.print_bottom_line(printer, addr);
     }
 
     /// moves the cursor xdiff down and ydiff to the right,
@@ -122,27 +110,30 @@ impl Unaligned {
         self.set_cursor(printer, CursorActive::None);
         // update cursor if updating the cursor is wanted, otherwise only change the view position
         let diff = if cursor {
-            self.cursor.move_cursor(xdiff, ydiff)
+            self.dh.cursor.move_cursor(xdiff, ydiff)
         } else {
-            self.cursor.get_size_x() as isize * ydiff + xdiff
+            self.dh.cursor.get_size_x() as isize * ydiff + xdiff
         };
         // update the compvec in case the views are moved independently
         let index_diff = match self.cursor_act {
             CursorActive::Both => diff,
-            CursorActive::Left => self.data.add_left_shift(-diff),
-            CursorActive::Right => self.data.add_right_shift(-diff),
+            CursorActive::First => self.data.add_first_shift(-diff),
+            CursorActive::Second => self.data.add_second_shift(-diff),
             CursorActive::None => diff,
         };
         self.index += index_diff;
         // if they are moved independently, we cannot scroll
         if !matches!(self.cursor_act, CursorActive::Both) {
             self.redraw(printer, false);
-        } else if let Some(scroll_amount) = self.cursor.full_row_move(index_diff) {
+        } else if let Some(scroll_amount) = self.dh.cursor.full_row_move(index_diff) {
             // scroll if we can
             let content = self.get_content();
-            print_doublehex_scrolled(&content, printer, self.cursor.get_size_y(), scroll_amount);
+            self.dh
+                .print_doublehex_scrolled(&content, printer, scroll_amount);
             self.set_cursor(printer, self.cursor_act);
-            self.print_bars(printer);
+            if scroll_amount != 0 {
+                self.print_bars(printer);
+            }
             printer.refresh();
         } else {
             self.redraw(printer, false);
@@ -160,11 +151,14 @@ impl Unaligned {
             Action::Right => self.move_around(printer, 1, 0, true),
             Action::RightAlt => self.move_around(printer, 1, 0, false),
             Action::PgDown => {
-                self.move_around(printer, 0, self.cursor.get_size_y() as isize / 2, false)
-            }
-            Action::PgUp => {
-                self.move_around(printer, 0, -(self.cursor.get_size_y() as isize) / 2, false)
+                self.move_around(printer, 0, self.dh.cursor.get_size_y() as isize / 2, false)
             }
+            Action::PgUp => self.move_around(
+                printer,
+                0,
+                -(self.dh.cursor.get_size_y() as isize) / 2,
+                false,
+            ),
             Action::NextDifference => self.jump_next_difference(printer),
             _ => (),
         }
@@ -173,9 +167,9 @@ impl Unaligned {
     pub fn process_action<B: Backend>(&mut self, printer: &mut B, action: Action) {
         match action {
             Action::Refresh => self.refresh(printer),
-            Action::CursorLeft => {
-                self.cursor_act = CursorActive::Left;
-                self.set_cursor(printer, CursorActive::Left);
+            Action::CursorFirst => {
+                self.cursor_act = CursorActive::First;
+                self.set_cursor(printer, CursorActive::First);
                 printer.refresh()
             }
             Action::CursorBoth => {
@@ -183,21 +177,21 @@ impl Unaligned {
                 self.set_cursor(printer, CursorActive::Both);
                 printer.refresh()
             }
-            Action::CursorRight => {
-                self.cursor_act = CursorActive::Right;
-                self.set_cursor(printer, CursorActive::Right);
+            Action::CursorSecond => {
+                self.cursor_act = CursorActive::Second;
+                self.set_cursor(printer, CursorActive::Second);
                 printer.refresh()
             }
             otherwise => self.process_move(printer, otherwise),
         }
     }
     pub fn goto_index<B: Backend>(&mut self, printer: &mut B, index: isize) {
-        let address_diff = index - (self.index + self.cursor.get_index() as isize);
-        let (col, row) = self.cursor.jump(address_diff);
+        let address_diff = index - (self.index + self.dh.cursor.get_index() as isize);
+        let (col, row) = self.dh.cursor.jump(address_diff);
         self.move_around(printer, 0, row, false);
         self.move_around(printer, col, 0, true);
     }
-    /// Go to the address in `pos`, right is true if on the right view, else the left view is used.
+    /// Go to the address in `pos`, right is true if on the second view, else the first view is used.
     /// Returns true if the address exists.
     pub fn goto<B: Backend>(
         &mut self,
@@ -205,13 +199,13 @@ impl Unaligned {
         right: bool,
         pos: usize,
     ) -> Result<(), String> {
-        if !right && !self.cursor_act.is_left() {
+        if !right && !self.cursor_act.is_first() {
             return Err(
-                "Attempting to search on left side, but current cursor is on right side".into(),
+                "Attempting to search on first view, but current cursor is on second view".into(),
             );
-        } else if right && !self.cursor_act.is_right() {
+        } else if right && !self.cursor_act.is_second() {
             return Err(
-                "Attempting to search on right side, but current cursor is on left side".into(),
+                "Attempting to search on second view, but current cursor is on first view".into(),
             );
         }
         self.goto_index(
@@ -223,8 +217,8 @@ impl Unaligned {
     pub fn jump_next_difference<B: Backend>(&mut self, printer: &mut B) {
         // skip half a page
         let first_address = (self.index
-            + self.cursor.get_index() as isize
-            + (self.cursor.get_size_y() / 2) as isize * self.cursor.get_size_x() as isize)
+            + self.dh.cursor.get_index() as isize
+            + (self.dh.cursor.get_size_y() / 2) as isize * self.dh.cursor.get_size_x() as isize)
             .clamp(self.data.bounds().start, self.data.bounds().end - 1);
         let mut target_address = first_address;
         for i in first_address..self.data.bounds().end {
@@ -237,13 +231,13 @@ impl Unaligned {
         self.goto_index(printer, target_address);
     }
     /// Turns the view into most of its parts
-    pub fn destruct(self) -> Result<(PointedFile, PointedFile, CursorState), Self> {
-        let cursor_index = self.index + self.cursor.get_index() as isize;
+    pub fn destruct(self) -> Result<(PointedFile, PointedFile, DoubleHexContext), Self> {
+        let cursor_index = self.index + self.dh.cursor.get_index() as isize;
         // for now we only return if the cursor is at a positions where both indexes are actually
         // inside the file
         if let (Some(laddr), Some(raddr)) = (
-            self.data.get_left_addr(cursor_index),
-            self.data.get_right_addr(cursor_index),
+            self.data.get_first_addr(cursor_index),
+            self.data.get_second_addr(cursor_index),
         ) {
             let (lvec, rvec) = self.data.get_data();
             Ok((
@@ -257,7 +251,7 @@ impl Unaligned {
                     content: rvec,
                     index: raddr,
                 },
-                self.cursor,
+                self.dh,
             ))
         } else {
             Err(self)
@@ -283,9 +277,9 @@ impl From<Action> for AlignedMessage {
 pub struct Aligned {
     data: DoubleVec<AlignElement>,
     filenames: (String, String),
-    original: (Arc<Vec<u8>>, Arc<Vec<u8>>),
+    original: (FileContent, FileContent),
     index: isize,
-    cursor: CursorState,
+    pub dh: DoubleHexContext,
 }
 
 impl Aligned {
@@ -294,37 +288,37 @@ impl Aligned {
     /// Note that receiving events and sending them to the view has to be handled by
     /// the caller for unknown reasons.
     pub fn new(
-        left: PointedFile,
-        right: PointedFile,
-        cursor: CursorState,
+        first: PointedFile,
+        second: PointedFile,
+        dh: DoubleHexContext,
         algo: &AlignAlgorithm,
         sender: Sender<AlignedMessage>,
     ) -> Self {
-        let index = -(cursor.get_index() as isize);
+        let index = -(dh.cursor.get_index() as isize);
         let data = DoubleVec::new();
-        let left_arc = left.content.clone();
-        let right_arc = right.content.clone();
-        algo.start_align(left_arc, right_arc, (left.index, right.index), sender);
+        let first_arc = first.content.clone();
+        let second_arc = second.content.clone();
+        algo.start_align(first_arc, second_arc, (first.index, second.index), sender);
         Aligned {
             data,
-            filenames: (left.name, right.name),
-            original: (left.content, right.content),
+            filenames: (first.name, second.name),
+            original: (first.content, second.content),
             index,
-            cursor,
+            dh,
         }
     }
     /// Checks whether a given range of indexes overlaps with the indexes currently visible.
     fn is_in_view(&self, range: Range<isize>) -> bool {
-        let self_range =
-            self.index..self.index + (self.cursor.get_size_x() * self.cursor.get_size_y()) as isize;
+        let self_range = self.index
+            ..self.index + (self.dh.cursor.get_size_x() * self.dh.cursor.get_size_y()) as isize;
         !(self_range.start >= range.end || self_range.end <= range.start)
     }
     /// Gets a useful form of the information contained in the alignement data for printing.
     fn get_content(&self) -> Vec<DoubleHexLine> {
         let mut content = Vec::new();
-        for x in 0..self.cursor.get_size_y() {
+        for x in 0..self.dh.cursor.get_size_y() {
             // address of current line to be converted
-            let base_addr = (x * self.cursor.get_size_x()) as isize + self.index;
+            let base_addr = (x * self.dh.cursor.get_size_x()) as isize + self.index;
             let address = self
                 .data
                 .get(base_addr)
@@ -332,7 +326,7 @@ impl Aligned {
                 .unwrap_or_default();
             let bytes = self
                 .data
-                .get_range(base_addr..base_addr + self.cursor.get_size_x() as isize)
+                .get_range(base_addr..base_addr + self.dh.cursor.get_size_x() as isize)
                 .iter()
                 .map(|malignel| {
                     malignel
@@ -346,32 +340,32 @@ impl Aligned {
     }
     /// Paints the cursor at the current position
     fn set_cursor<B: Backend>(&self, printer: &mut B, cursor_act: CursorActive) {
-        let cursor_index = self.index + self.cursor.get_index() as isize;
-        set_doublehex_cursor(
-            printer,
-            &self.cursor,
-            cursor_act,
-            self.data
-                .get(cursor_index)
-                .map(|alignel| (alignel.xbyte, alignel.ybyte))
-                .unwrap_or_default(),
-        );
+        let cursor_index = self.index + self.dh.cursor.get_index() as isize;
+        let bytes = self
+            .data
+            .get(cursor_index)
+            .map(|alignel| (alignel.xbyte, alignel.ybyte))
+            .unwrap_or_default();
+        let addresses = self
+            .data
+            .get(cursor_index)
+            .map(|alignel| (Some(alignel.xaddr), Some(alignel.yaddr)))
+            .unwrap_or_default();
+        self.dh
+            .set_doublehex_cursor(printer, cursor_act, bytes, addresses);
     }
 
     /// Prints the top and bottom bar.
     fn print_bars<B: Backend>(&self, printer: &mut B) {
-        print_title_line(
-            printer,
-            " aligned",
-            &self.filenames.0,
-            &self.filenames.1,
-            self.cursor.get_size_x(),
-        );
-        print_bottom_line(
-            printer,
-            self.cursor.get_size_x(),
-            self.cursor.get_size_y() + 1,
-        );
+        self.dh
+            .print_title_line(printer, " aligned", &self.filenames.0, &self.filenames.1);
+        let cursor_index = self.index + self.dh.cursor.get_index() as isize;
+        let addresses = self
+            .data
+            .get(cursor_index)
+            .map(|alignel| (Some(alignel.xaddr), Some(alignel.yaddr)))
+            .unwrap_or_default();
+        self.dh.print_bottom_line(printer, addresses);
     }
 
     /// Moves the cursor xdiff down and ydiff to the right,
@@ -385,16 +379,19 @@ impl Aligned {
     ) {
         self.set_cursor(printer, CursorActive::None);
         let index_diff = if cursor {
-            self.cursor.move_cursor(xdiff, ydiff)
+            self.dh.cursor.move_cursor(xdiff, ydiff)
         } else {
-            self.cursor.get_size_x() as isize * ydiff + xdiff
+            self.dh.cursor.get_size_x() as isize * ydiff + xdiff
         };
         self.index += index_diff;
-        if let Some(scroll_amount) = self.cursor.full_row_move(index_diff) {
+        if let Some(scroll_amount) = self.dh.cursor.full_row_move(index_diff) {
             let content = self.get_content();
-            print_doublehex_scrolled(&content, printer, self.cursor.get_size_y(), scroll_amount);
+            self.dh
+                .print_doublehex_scrolled(&content, printer, scroll_amount);
             self.set_cursor(printer, CursorActive::Both);
-            self.print_bars(printer);
+            if scroll_amount != 0 {
+                self.print_bars(printer);
+            }
             printer.refresh();
         } else {
             self.redraw(printer, false);
@@ -417,10 +414,9 @@ impl Aligned {
     /// Resize the view without printing it, returns whether redrawing is necessary.
     pub fn resize(&mut self, dimensions: (usize, usize)) -> bool {
         let (columns, rows) = dimensions;
-        let hex_col = get_doublehex_columns(columns);
-        let old_dimensions = (self.cursor.get_size_x(), self.cursor.get_size_y());
-        let new_dimensions = (hex_col, rows - 2);
-        self.index += self.cursor.resize(new_dimensions);
+        let old_dimensions = (self.dh.cursor.get_size_x(), self.dh.cursor.get_size_y());
+        let new_dimensions = self.dh.style.get_doublehex_dims(columns, rows);
+        self.index += self.dh.cursor.resize(new_dimensions);
         (old_dimensions) != new_dimensions
     }
     /// Redraws the current view without checking and updating the view for changes.
@@ -429,7 +425,7 @@ impl Aligned {
             printer.clear();
         }
         let content = self.get_content();
-        print_doublehex_screen(&content, printer);
+        self.dh.print_doublehex_screen(&content, printer);
         self.set_cursor(printer, CursorActive::Both);
         self.print_bars(printer);
         printer.refresh();
@@ -440,12 +436,12 @@ impl Aligned {
         self.redraw(printer, changed);
     }
     pub fn goto_index<B: Backend>(&mut self, printer: &mut B, index: isize) {
-        let address_diff = index - (self.index + self.cursor.get_index() as isize);
-        let (col, row) = self.cursor.jump(address_diff);
+        let address_diff = index - (self.index + self.dh.cursor.get_index() as isize);
+        let (col, row) = self.dh.cursor.jump(address_diff);
         self.move_around(printer, 0, row, false);
         self.move_around(printer, col, 0, true);
     }
-    /// Go to the address in `pos`, right is true if on the right view, else the left view is used.
+    /// Go to the address in `pos`, right is true if on the second view, else the first view is used.
     /// Returns true if the address exists.
     pub fn goto<B: Backend>(
         &mut self,
@@ -465,8 +461,8 @@ impl Aligned {
     pub fn jump_next_difference<B: Backend>(&mut self, printer: &mut B) {
         // skip half a page
         let first_address = (self.index
-            + self.cursor.get_index() as isize
-            + (self.cursor.get_size_y() / 2) as isize * self.cursor.get_size_x() as isize)
+            + self.dh.cursor.get_index() as isize
+            + (self.dh.cursor.get_size_y() / 2) as isize * self.dh.cursor.get_size_x() as isize)
             .clamp(self.data.bounds().start, self.data.bounds().end - 1);
         let mut target_address = first_address;
         for i in first_address..=self.data.bounds().end {
@@ -490,11 +486,14 @@ impl Aligned {
             Action::Right => self.move_around(printer, 1, 0, true),
             Action::RightAlt => self.move_around(printer, 1, 0, false),
             Action::PgDown => {
-                self.move_around(printer, 0, self.cursor.get_size_y() as isize / 2, false)
-            }
-            Action::PgUp => {
-                self.move_around(printer, 0, -(self.cursor.get_size_y() as isize) / 2, false)
+                self.move_around(printer, 0, self.dh.cursor.get_size_y() as isize / 2, false)
             }
+            Action::PgUp => self.move_around(
+                printer,
+                0,
+                -(self.dh.cursor.get_size_y() as isize) / 2,
+                false,
+            ),
             Action::NextDifference => self.jump_next_difference(printer),
             _ => (),
         }
@@ -521,10 +520,12 @@ impl Aligned {
         }
     }
     /// Turn an Aligned view into its part, including information on where it points
-    pub fn destruct(self) -> Result<(PointedFile, PointedFile, CursorState), Self> {
+    pub fn destruct(self) -> Result<(PointedFile, PointedFile, DoubleHexContext), Self> {
         // we return the original view in case the cursor is outside the files
-        match (self.data.get(self.index + self.cursor.get_index() as isize))
-            .map(|a| (a.xaddr, a.yaddr))
+        match (self
+            .data
+            .get(self.index + self.dh.cursor.get_index() as isize))
+        .map(|a| (a.xaddr, a.yaddr))
         {
             Some((xaddr, yaddr)) => Ok((
                 PointedFile {
@@ -537,7 +538,7 @@ impl Aligned {
                     content: self.original.1,
                     index: yaddr,
                 },
-                self.cursor,
+                self.dh,
             )),
             None => Err(self),
         }
